import pandas as pd
import numpy as np

data = pd.read_csv("data/DATASET_CMA_CGM_NERVAL_5min.csv")
data = data.iloc[:,:-1] # afairei tin teleutaia stili TIMESTAMP
columns = data.columns
data = np.array(data)

# CUSUM

# CALCULATE THE AVERAGE OF EACH STREAM AS TARGET VALUE
average = np.zeros(len(data[0]))
for i in range(0,10):
	for col in range(0, len(data[0])):
		average[col] += data[i, col]

average = average/10
# CONSTANTS
# m = average
m = abs(average)
# print(m)
kpos = kneg = m/2
thpos = thneg = 2*m

# f = open('test','a')

# INITIALIZE VECTORS
P = np.zeros(len(data[0]))	#positive changes
N = np.zeros(len(data[0]))	#negative changes
s = np.zeros((len(data), len(data[0])))

for t in range(0,len(data)):
	for col in range(0,len(data[0])):

		spos = 0	#positive signal
		sneg = 0	#negative signal
		P[col] = max(0, abs(data[t,col]) - m[col] - kpos[col] + P[col])
		N[col] = min(0, abs(data[t,col]) - m[col] + kneg[col] + N[col])
		# f.write("iter "+ str(t)+" "+str(col)+" "+str(P[col])+" "+str(N[col])+" threshpos "+str(thpos[col])+" threshneg "+str(thneg[col])+'\n')
		if (P[col] > thpos[col]):
			spos = 1
			P[col] = N[col] = 0
		if (N[col] < -thneg[col]):
			sneg = 1
			P[col] = N[col] = 0
		s[t,col] = spos or sneg

pd.DataFrame(s, columns=columns).to_csv('data/cumsum_script.csv')

# f.close()
# np.savetxt('../eventVectors/cusum/cusumEventVector.csv', s, fmt='%d', delimiter='')


import time
from math import exp
import pandas as pd
import numpy as np


from sklearn.neighbors import KNeighborsClassifier

def powerset(lst):
    # the power set of the empty set has one element, the empty set
	result = lst
	res = list(map(list, result))
	return res

# k = ageing factor, i = how many steps is the prob active
def linearAgeing(i):
	age = 1 - 0.05*i
	return age

def exponentialAgeing(i):
	k = 0.05
	age = exp(-k*i)
	return age


def updateFreq(event, freq):
	if event not in freq:
		freq[event] = 0
	freq[event] += 1


def updateGraph(prevState, event, graph):
	if event not in graph:
		graph[event] = {}

	if event not in graph[prevState]:
		graph[prevState][event] = 0
	graph[prevState][event] += 1

class EventCorrelationClassifier(KNeighborsClassifier):
    pass


import pandas as pd
import numpy as np
import math

data = pd.read_csv("DATASET_CMA_CGM_NERVAL_5min.csv")
data = data.iloc[:,:-1]
columns = data.columns
data = np.array(data)

# SHEWHART

# CONSTANTS
# k = data[0]
k = 1

# INITIALIZE VECTORS
xprev = np.zeros(len(data[0]))
sprev = np.zeros(len(data[0]))
xcurr = np.zeros(len(data[0]))
scurr = np.zeros(len(data[0]))
s = np.zeros((len(data), len(data[0])))

for t in range(0,len(data)):
	for col in range(0,len(data[0])):

		xcurr[col] = xprev[col] + (data[t,col]-xprev[col])/(t+1)
		scurr[col] = math.sqrt( (1/(t+1)) * (t*(sprev[col]**2) + (data[t,col]-xcurr[col])*(data[t,col]-xprev[col]) ))

		UCL = xcurr[col] + k*scurr[col]
		LCL = xcurr[col] - k*scurr[col]

		if ((data[t,col] > UCL) or (data[t,col] < LCL)):
			s[t][col] = 1
		else:
			s[t][col] = 0
		xprev[col] = xcurr[col]
		sprev[col] = scurr[col]

# np.savetxt('../eventVectors/shewhart/shewhartEventVector.csv', s, fmt='%d', delimiter='')

# pd.DataFrame(s, columns=columns).to_csv('data/shewhart_script.csv')